@@ -1,12 +1,12 @@
 /**
  *  @File     pluginapids.h
  *
- *  @Author   Lukas Zeller (luz@synthesis.ch)
+ *  @Author   Lukas Zeller (luz@plan44.ch)
  *
  *  @brief TPluginApiDS
  *    Plugin based datastore API implementation
  *
- *    Copyright (c) 2001-2009 by Synthesis AG (www.synthesis.ch)
+ *    Copyright (c) 2001-2011 by Synthesis AG + plan44.ch
  *
  *  @Date 2005-10-06 : luz : created from apidbdatastore
  */
@@ -106,6 +106,8 @@ public:
   /// either  fDBAPIModule_Data or fDBAPIModule_Admin to select plugin handling of
   /// either data or admin independently.
   bool fDataModuleAlsoHandlesAdmin;
+  // wants startDataRead() called as early as possible
+  bool fEarlyStartDataRead;
   // - config object for API module
   TDB_Api_Config fDBApiConfig_Data;
   TDB_Api_Config fDBApiConfig_Admin;
@@ -116,7 +118,8 @@ public:
   uInt16 fPluginDbgMask_Data;
   uInt16 fPluginDbgMask_Admin;
   // capabilities of connected plugin
-  bool fItemAsKey;
+  bool fItemAsKey; // supports items as key
+  bool fHasDeleteSyncSet; // implements deleting sync set using DeleteSyncSet()
   // public methods
   // - create appropriate datastore from config, calls addTypeSupport as well
   virtual TLocalEngineDS *newLocalDataStore(TSyncSession *aSessionP);
@@ -169,6 +172,10 @@ public:
   virtual void dsResetDataStore(void) { InternalResetDataStore(); inherited::dsResetDataStore(); };
   virtual ~TPluginApiDS();
 
+  // override TSyncDataStore: the plugin must be able to return 404
+  // when an item is not found during delete
+  virtual bool dsDeleteDetectsItemPresence() const { return true; }
+
   #ifndef BINFILE_ALWAYS_ACTIVE
   /// @name apiXXXX methods defining the interface from TCustomImplDS to TXXXApi actual API implementations
   /// @{
@@ -251,6 +258,9 @@ public:
   ///   them separately afterwards).
   #endif // not BINFILE_ALWAYS_ACTIVE
 
+  /// perform early data access start (if datastore requests it by setting fEarlyStartDataRead config flag)
+  virtual localstatus apiEarlyDataAccessStart(void);
+  /// read the sync set
   virtual localstatus apiReadSyncSet(bool aNeedAll);
   /// Zap all data in syncset (note that everything outside the sync set will remain intact)
   virtual localstatus apiZapSyncSet(void);
@@ -313,6 +323,8 @@ public:
 private:
   // - connect data handling part of plugin. Returns LOCERR_NOTIMPL when no data plugin is selected
   TSyError connectDataPlugin(void);
+  // - prepare for reading syncset (is called early when fEarlyStartDataRead is set, otherwise from within apiReadSyncSet)
+  localstatus apiPrepareReadSyncSet(void);
   // - alert possible thread change to plugins
   //   Does not check if API is locked or not, see dsThreadMayChangeNow()
   void ThreadMayChangeNow(void);
@@ -367,6 +379,9 @@ private:
   TDB_Api fDBApi_Admin; ///< access to admin
   // config pointer
   TPluginDSConfig *fPluginDSConfigP;
+  // filter testing
+	bool fAPICanFilter;
+	bool fAPIFiltersTested;  
 }; // TPluginApiDS
 
 
