@@ -1,11 +1,11 @@
 /*
  *  File:         debuglogger.cpp
  *
- *  Author:       Lukas Zeller (luz@synthesis.ch)
+ *  Author:       Lukas Zeller (luz@plan44.ch)
  *
  *  Global debug mechanisms
  *
- *  Copyright (c) 2005-2009 by Synthesis AG (www.synthesis.ch)
+ *  Copyright (c) 2005-2011 by Synthesis AG + plan44.ch
  *
  *  2005-08-04 : luz : created
  *
@@ -44,6 +44,14 @@ cAppCharP const DbgFoldingModeNames[numDbgFoldingModes] = {
 };
 
 
+cAppCharP const DbgSourceModeNames[numDbgSourceModes] = {
+  "none",       // do not include links into source code in HTML logs
+  "hint",       // no links, but info about what file/line number the message comes from 
+  "doxygen",    // include link into doxygen prepared HTML version of source code
+  "txmt",       // include txmt:// link (understood by TextMate and BBEdit) into source code
+};
+
+
 // debug flush modes
 cAppCharP const DbgFlushModeNames[numDbgFlushModes] = {
   "buffered",   // no flush, keep open as long as possible, output buffered (fast, needed for network drives)
@@ -217,19 +225,23 @@ TDbgOptions::TDbgOptions()
 
 void TDbgOptions::clear(void)
 {
-  fOutputFormat=dbgfmt_html; // most universally readable and convenient
-  fIndentString="  "; // two spaces
+  fOutputFormat = dbgfmt_html; // most universally readable and convenient
+  fIndentString = "  "; // two spaces
   fCustomPrefix.erase(); // no custom prefix
   fCustomSuffix.erase(); // no custom suffix
-  fSeparateMsgs=true; // separate text message lines (<msg></msg> in xml)
-  fTimestampStructure=true; // timestamps in structure...
-  fTimestampForAll=false; // ..but not for every line
-  fThreadIDForAll=false; // not by default
-  fFlushMode=dbgflush_none; // no special flush or openclose (fast, but might loose info on process abort)
-  fFoldingMode=dbgfold_auto; // dynamic folding enabled, expanded/collapsed defaults automatically set on block-by-block basis
-  fAppend=false; // default to overwrite existing logfiles
-  fSubThreadMode=dbgsubthread_suppress; // simply suppress subthread info
-  fSubThreadBufferMax=1024*1024; // don't buffer more than one meg.
+  fSeparateMsgs = true; // separate text message lines (<msg></msg> in xml)
+  fTimestampStructure = true; // timestamps in structure...
+  fTimestampForAll = false; // ..but not for every line
+  fThreadIDForAll = false; // not by default
+  fFlushMode = dbgflush_none; // no special flush or openclose (fast, but might loose info on process abort)
+  fFoldingMode = dbgfold_auto; // dynamic folding enabled, expanded/collapsed defaults automatically set on block-by-block basis
+  #ifdef SYDEBUG_LOCATION
+  fSourceLinkMode = dbgsource_none; // no links into source code
+  fSourceRootPath = SYDEBUG_LOCATION; // use default path from build
+  #endif
+  fAppend = false; // default to overwrite existing logfiles
+  fSubThreadMode = dbgsubthread_suppress; // simply suppress subthread info
+  fSubThreadBufferMax = 1024*1024; // don't buffer more than one meg.
 } // TDbgOptions::clear
 
 
@@ -358,16 +370,17 @@ void TStdFileDbgOut::putLine(cAppCharP aLine, bool aForceFlush)
       // now output
       fputs(aLine,fFile);
       fputs("\n",fFile);
-    }
-    // do required flushing
-    if (fFlushMode==dbgflush_openclose) {
-      // we need to close the file after every line of output
-      fclose(fFile);
-      fFile=NULL;
-    }
-    else if (aForceFlush || fFlushMode==dbgflush_flush) {
-      // simply flush
-      fflush(fFile);
+
+      // do required flushing
+      if (fFlushMode==dbgflush_openclose) {
+        // we need to close the file after every line of output
+        fclose(fFile);
+        fFile=NULL;
+      }
+      else if (aForceFlush || fFlushMode==dbgflush_flush) {
+        // simply flush
+        fflush(fFile);
+      }
     }
   }
 } // TStdFileDbgOut::putLine
@@ -470,7 +483,7 @@ TDebugLoggerBase::~TDebugLoggerBase()
 {
   // make sure debug is finalized
   DebugFinalizeOutput();
-  // make sure eventually left-over history elements are erased
+  // make sure possibly left-over history elements are erased
   while (fBlockHistory) {
     TBlockLevel *bl=fBlockHistory;
     fBlockHistory=bl->fNext;
@@ -492,7 +505,7 @@ lineartime_t TDebugLoggerBase::getSystemNowAs(timecontext_t aContext)
 // install outputter
 void TDebugLoggerBase::installOutput(TDbgOut *aDbgOutP)
 {
-  // get rid of eventually installed previous outputter
+  // get rid of possibly installed previous outputter
   if (fDbgOutP) delete fDbgOutP;
   fDbgOutP=aDbgOutP;
 } // TDebugLoggerBase::installOutput
@@ -509,7 +522,7 @@ void TDebugLoggerBase::outputVia(TDebugLoggerBase *aDebugLoggerP)
 
 
 // output formatted text
-void TDebugLoggerBase::DebugVPrintf(uInt32 aDbgMask, cAppCharP aFormat, va_list aArgs)
+void TDebugLoggerBase::DebugVPrintf(TDBG_LOCATION_PROTO uInt32 aDbgMask, cAppCharP aFormat, va_list aArgs)
 {
   // we need a format and debug not completely off
   if ((getMask() & aDbgMask)==aDbgMask && aFormat) {
@@ -519,7 +532,7 @@ void TDebugLoggerBase::DebugVPrintf(uInt32 aDbgMask, cAppCharP aFormat, va_list
     // assemble the message string
     vsnprintf(msg, maxmsglen, aFormat, aArgs);
     // write the string
-    DebugPuts(aDbgMask,msg);
+    DebugPuts(TDBG_LOCATION_ARG aDbgMask,msg);
   }
 } // TDebugLoggerBase::DebugVPrintf
 
@@ -533,13 +546,13 @@ TDebugLoggerBase &TDebugLoggerBase::setNextMask(uInt32 aDbgMask)
 
 
 // like DebugPrintf(), but using mask previously set by setNextMask()
-void TDebugLoggerBase::DebugPrintfLastMask(cAppCharP aFormat, ...)
+void TDebugLoggerBase::DebugPrintfLastMask(TDBG_LOCATION_PROTO cAppCharP aFormat, ...)
 {
   va_list args;
   // we need a format and debug not completely off
   if ((getMask() & fNextDebugMask)==fNextDebugMask && aFormat) {
     va_start(args, aFormat);
-    DebugVPrintf(fNextDebugMask,aFormat,args);
+    DebugVPrintf(TDBG_LOCATION_ARG fNextDebugMask,aFormat,args);
     va_end(args);
   }
   fNextDebugMask=0;
@@ -547,79 +560,177 @@ void TDebugLoggerBase::DebugPrintfLastMask(cAppCharP aFormat, ...)
 
 
 // output formatted text
-void TDebugLoggerBase::DebugPrintf(uInt32 aDbgMask, cAppCharP aFormat, ...)
+void TDebugLoggerBase::DebugPrintf(TDBG_LOCATION_PROTO uInt32 aDbgMask, cAppCharP aFormat, ...)
 {
   va_list args;
   // we need a format and debug not completely off
   if ((getMask() & aDbgMask)==aDbgMask && aFormat) {
     va_start(args, aFormat);
-    DebugVPrintf(aDbgMask,aFormat,args);
+    DebugVPrintf(TDBG_LOCATION_ARG aDbgMask,aFormat,args);
     va_end(args);
   }
 } // TDebugLoggerBase::DebugVPrintf
 
 
+// open new Block without attribute list
+void TDebugLoggerBase::DebugOpenBlock(TDBG_LOCATION_PROTO cAppCharP aBlockName, cAppCharP aBlockTitle, bool aCollapsed)
+{
+  // we need a format and debug not completely off
+  if (getMask() && aBlockName) {
+#ifdef __clang__
+    #pragma clang diagnostic push
+    #pragma clang diagnostic ignored "-Wformat-security"
+#endif
+    DebugOpenBlock(TDBG_LOCATION_ARG aBlockName,aBlockTitle,aCollapsed,NULL);
+#ifdef __clang__
+    #pragma clang diagnostic pop
+#endif
+  }
+} // TDebugLoggerBase::DebugOpenBlock
+
+
 // open new Block with attribute list, printf style
-void TDebugLoggerBase::DebugOpenBlock(cAppCharP aBlockName, cAppCharP aBlockTitle, bool aCollapsed, cAppCharP aBlockFmt, ...)
+void TDebugLoggerBase::DebugOpenBlock(TDBG_LOCATION_PROTO cAppCharP aBlockName, cAppCharP aBlockTitle, bool aCollapsed, cAppCharP aBlockFmt, ...)
 {
   va_list args;
   // we need a format and debug not completely off
   if (getMask() && aBlockName) {
     va_start(args, aBlockFmt);
-    DebugVOpenBlock(aBlockName,aBlockTitle,aCollapsed,aBlockFmt,args);
+    DebugVOpenBlock(TDBG_LOCATION_ARG aBlockName,aBlockTitle,aCollapsed,aBlockFmt,args);
     va_end(args);
   }
 } // TDebugLoggerBase::DebugOpenBlock
 
 
 // open new Block with attribute list, printf style, expanded by default
-void TDebugLoggerBase::DebugOpenBlockExpanded(cAppCharP aBlockName, cAppCharP aBlockTitle, cAppCharP aBlockFmt, ...)
+void TDebugLoggerBase::DebugOpenBlockExpanded(TDBG_LOCATION_PROTO cAppCharP aBlockName, cAppCharP aBlockTitle, cAppCharP aBlockFmt, ...)
 {
   va_list args;
   // we need a format and debug not completely off
   if (getMask() && aBlockName) {
     va_start(args, aBlockFmt);
-    DebugVOpenBlock(aBlockName,aBlockTitle,false,aBlockFmt,args);
+    DebugVOpenBlock(TDBG_LOCATION_ARG aBlockName,aBlockTitle,false,aBlockFmt,args);
     va_end(args);
   }
 } // TDebugLoggerBase::DebugOpenBlockExpanded
 
 
 // open new Block with attribute list, printf style, collapsed by default
-void TDebugLoggerBase::DebugOpenBlockCollapsed(cAppCharP aBlockName, cAppCharP aBlockTitle, cAppCharP aBlockFmt, ...)
+void TDebugLoggerBase::DebugOpenBlockCollapsed(TDBG_LOCATION_PROTO cAppCharP aBlockName, cAppCharP aBlockTitle, cAppCharP aBlockFmt, ...)
 {
   va_list args;
   // we need a format and debug not completely off
   if (getMask() && aBlockName) {
     va_start(args, aBlockFmt);
-    DebugVOpenBlock(aBlockName,aBlockTitle,true,aBlockFmt,args);
+    DebugVOpenBlock(TDBG_LOCATION_ARG aBlockName,aBlockTitle,true,aBlockFmt,args);
     va_end(args);
   }
 } // TDebugLoggerBase::DebugOpenBlockCollapsed
 
 
-// open new Block without attribute list
-void TDebugLoggerBase::DebugOpenBlock(cAppCharP aBlockName, cAppCharP aBlockTitle, bool aCollapsed)
+#ifdef SYDEBUG_LOCATION
+
+#define MAKEDBGLINK(txt) dbg2Link(TDBG_LOCATION_ARG txt)
+
+/// turn text into link to source code
+string TDebugLoggerBase::dbg2Link(const TDbgLocation &aTDbgLoc, const string &aTxt)
 {
-  // we need a format and debug not completely off
-  if (getMask() && aBlockName) {
-    static va_list va;
-    DebugVOpenBlock(aBlockName,aBlockTitle,aCollapsed,NULL,va);
-  }
-} // TDebugLoggerBase::DebugOpenBlock
+  if (!aTDbgLoc.fFile || !fDbgOptionsP || fDbgOptionsP->fSourceLinkMode==dbgsource_none || fDbgOptionsP->fOutputFormat!=dbgfmt_html)
+    return aTxt; // disabled, non-html or no information to create source link
+	// create link or hint to source code
+  string line;
+
+	switch(fDbgOptionsP->fSourceLinkMode) {
+		case dbgsource_hint: {
+    	// only add name/line number/function as title hint (in a otherwise inactive link)
+      line = "<a href=\"#\" title=";
+      StringObjPrintf(line,"<a href=\"#\" title=\"%s:%d",aTDbgLoc.fFile,aTDbgLoc.fLine);
+      StringObjAppendPrintf(line," in %s",aTDbgLoc.fFunction);
+      line += '"';  
+      goto closelink;
+    }
+		case dbgsource_doxygen: {
+    	// create link into doxygen
+      line = "<a href=\"";
+      // replace path with path to Doxygen HTML pages,
+      // mangle base name like Doxygen does
+      line += fDbgOptionsP->fSourceRootPath;
+      line += "/";
+      string file = aTDbgLoc.fFile;
+      size_t off = file.rfind('/');
+      if (off != file.npos)
+        file = file.substr(off + 1);
+      for (off = 0; off < file.size(); off++) {
+        switch(file[off]) {
+        case '_':
+          line+="__";
+          break;
+        case '.':
+          line+="_8";
+          break;
+        default:
+          line+=file[off];
+          break;
+        }
+      }
+      StringObjAppendPrintf(line,"-source.html#l%05d",aTDbgLoc.fLine);
+      line+="\"";
+      if (aTDbgLoc.fFunction) {
+        line+=" title=\"";
+        line+=aTDbgLoc.fFunction;
+        line+="\"";
+      }
+      goto closelink;
+    }
+		case dbgsource_txmt: {
+    	// create txmt:// URL scheme link, which opens TextMate or BBEdit at the correct line in MacOS X
+      line = "<a href=\"txmt://open/?url=file://";
+      // - create path
+      string path = fDbgOptionsP->fSourceRootPath;
+      path += aTDbgLoc.fFile;
+      // - add path CGI encoded
+      line += encodeForCGI(path.c_str());
+      // - add line number
+      if (aTDbgLoc.fLine>0)
+	      StringObjAppendPrintf(line,"&line=%d",aTDbgLoc.fLine);  
+      line+="\"";
+      if (aTDbgLoc.fFunction) {
+        line+=" title=\"";
+        line+=aTDbgLoc.fFunction;
+        line+='"';
+      }
+    }
+    closelink: {
+      line+=">";
+      line+=aTxt;
+      line+="</a>";
+      break;
+    }
+  	default:
+    	line = aTxt;
+  } // switch
+  // return 
+  return line;
+} // TDebugLoggerBase::dbg2Link
+
+#else
+
+#define MAKEDBGLINK(txt) (txt)
+
+#endif // SYDEBUG_LOCATION
 
 
 // output text to debug channel
-void TDebugLoggerBase::DebugPuts(uInt32 aDbgMask, cAppCharP aText, stringSize aTextSize, bool aPreFormatted)
+void TDebugLoggerBase::DebugPuts(TDBG_LOCATION_PROTO uInt32 aDbgMask, cAppCharP aText, stringSize aTextSize, bool aPreFormatted)
 {
   // we need a text and debug not completely off
   if (!((getMask() & aDbgMask)==aDbgMask && aText && fDbgOptionsP)) {
     // cannot output
     //#ifdef __MWERKS__
     //#warning "ugly hack"
-    DebugPutLine("<li><span class=\"error\">Warning: Dbg output system already half shut down (limited formatting)!</span></li><li>");
-    if (aText) DebugPutLine(aText);
-    DebugPutLine("</li>");
+    DebugPutLine(TDBG_LOCATION_NONE "<li><span class=\"error\">Warning: Dbg output system already half shut down (limited formatting)!</span></li><li>");
+    if (aText) DebugPutLine(TDBG_LOCATION_NONE aText);
+    DebugPutLine(TDBG_LOCATION_NONE "</li>");
     //#endif
   }
   else {
@@ -658,19 +769,34 @@ void TDebugLoggerBase::DebugPuts(uInt32 aDbgMask, cAppCharP aText, stringSize aT
           if (firstLine) {
             line="<li>";
             // add timestamp if needed for every line
-            if (fDbgOptionsP->fTimestampForAll || fDbgOptionsP->fThreadIDForAll) {
-              line+="<i>[";
+            if (
+            	fDbgOptionsP->fTimestampForAll
+            	|| fDbgOptionsP->fThreadIDForAll
+              #ifdef SYDEBUG_LOCATION
+              || fDbgOptionsP->fSourceLinkMode!=dbgsource_none
+              #endif
+            ) {
+              string prefix;
+              prefix = "<i>[";
               #ifdef MULTI_THREAD_SUPPORT
               if (fDbgOptionsP->fThreadIDForAll) {
-                StringObjAppendPrintf(line,"%09lu",myThreadID());
-                if (fDbgOptionsP->fTimestampForAll) line+=", ";
+                StringObjAppendPrintf(prefix,"%09lu",myThreadID());
+                if (fDbgOptionsP->fTimestampForAll) prefix += ", ";
               }
               #endif
               if (fDbgOptionsP->fTimestampForAll) {
                 StringObjTimestamp(ts,getSystemNowAs(TCTX_SYSTEM));
-                line+=ts;
+                prefix += ts;
               }
-              line+="]</i>&nbsp;";
+              #ifdef SYDEBUG_LOCATION
+              else if (!fDbgOptionsP->fThreadIDForAll) {
+              	// neither threadID nor timestamp, but source requested -> put small text here
+                prefix += "src";
+              }
+              #endif
+              prefix+="]</i>&nbsp;";
+              // if we have links into source code, add it here
+              line += MAKEDBGLINK(prefix);
             }
             // colorize some messages
             string cl="";
@@ -749,7 +875,7 @@ void TDebugLoggerBase::DebugPuts(uInt32 aDbgMask, cAppCharP aText, stringSize aT
               line+=ts;
               line+="</time>";
             }
-            DebugPutLine(line.c_str(),line.size());
+            DebugPutLine(TDBG_LOCATION_NONE line.c_str(),line.size());
             line.erase();
           }
           if (fDbgOptionsP->fSeparateMsgs) {
@@ -868,7 +994,7 @@ void TDebugLoggerBase::DebugPuts(uInt32 aDbgMask, cAppCharP aText, stringSize aT
         }
         line+="</li>"; // we need to close the list entry
       }
-      DebugPutLine(line.c_str(),line.size(),pre);
+      DebugPutLine(TDBG_LOCATION_NONE line.c_str(),line.size(),pre);
       // next line, if any
       aText=p;
     } // loop until all text done
@@ -877,7 +1003,7 @@ void TDebugLoggerBase::DebugPuts(uInt32 aDbgMask, cAppCharP aText, stringSize aT
 
 
 // open new Block with attribute list, varargs passed
-void TDebugLoggerBase::DebugVOpenBlock(cAppCharP aBlockName, cAppCharP aBlockTitle, bool aCollapsed, cAppCharP aBlockFmt, va_list aArgs)
+void TDebugLoggerBase::DebugVOpenBlock(TDBG_LOCATION_PROTO cAppCharP aBlockName, cAppCharP aBlockTitle, bool aCollapsed, cAppCharP aBlockFmt, va_list aArgs)
 {
   if (!fDbgOptionsP)
     return;
@@ -891,7 +1017,7 @@ void TDebugLoggerBase::DebugVOpenBlock(cAppCharP aBlockName, cAppCharP aBlockTit
     // create Block line on current indent level
     string bl;
     string ts;
-    // - preamble, eventually with timestamp
+    // - preamble, possibly with timestamp
     bool withTime = fDbgOptionsP->fTimestampStructure;
     if (withTime)
       StringObjTimestamp(ts,getSystemNowAs(TCTX_SYSTEM));
@@ -920,8 +1046,13 @@ void TDebugLoggerBase::DebugVOpenBlock(cAppCharP aBlockName, cAppCharP aBlockTit
         }
         StringObjAppendPrintf(bl,"<a name=\"H%ld\">", long(getBlockNo()));
         if (withTime) {
-          bl+="[" + ts + "] ";
+          bl += MAKEDBGLINK(string("[") + ts + "] ");
         }
+        #ifdef SYDEBUG_LOCATION
+        else if (fDbgOptionsP->fSourceLinkMode!=dbgsource_none) {
+        	bl += MAKEDBGLINK(string("[src] "));
+        }
+        #endif
         bl+="'";
         bl+=aBlockName;
         bl+="'";
@@ -1035,7 +1166,7 @@ void TDebugLoggerBase::DebugVOpenBlock(cAppCharP aBlockName, cAppCharP aBlockTit
         break;
     } // switch preamble
     // now output Block line (on current indent level)
-    DebugPutLine(bl.c_str(), bl.size());
+    DebugPutLine(TDBG_LOCATION_NONE bl.c_str(), bl.size());
     // increase indent level (applies to all Block contents)
     fIndent++;
     // save Block on stack
@@ -1050,18 +1181,18 @@ void TDebugLoggerBase::DebugVOpenBlock(cAppCharP aBlockName, cAppCharP aBlockTit
 
 
 // close named Block. If no name given, topmost Block will be closed
-void TDebugLoggerBase::DebugCloseBlock(cAppCharP aBlockName)
+void TDebugLoggerBase::DebugCloseBlock(TDBG_LOCATION_PROTO cAppCharP aBlockName)
 {
   if (fOutStarted && getMask() && fDbgOptionsP && fBlockHistory) {
     if (aBlockName==NULL) {
       #if SYDEBUG>1
-      internalCloseBlocks(fBlockHistory->fBlockName.c_str(),"Block Nest Warning: Missing Block name at close");
+      internalCloseBlocks(TDBG_LOCATION_ARG fBlockHistory->fBlockName.c_str(),"Block Nest Warning: Missing Block name at close");
       #else
-      internalCloseBlocks(fBlockHistory->fBlockName.c_str(),NULL);
+      internalCloseBlocks(TDBG_LOCATION_ARG fBlockHistory->fBlockName.c_str(),NULL);
       #endif
     }
     else {
-      internalCloseBlocks(aBlockName,NULL);
+      internalCloseBlocks(TDBG_LOCATION_ARG aBlockName,NULL);
     }
   }
 } // TDebugLoggerBase::DebugCloseBlock
@@ -1069,7 +1200,7 @@ void TDebugLoggerBase::DebugCloseBlock(cAppCharP aBlockName)
 
 
 // internal helper used to close all or some Blocks
-void TDebugLoggerBase::internalCloseBlocks(cAppCharP aBlockName, cAppCharP aCloseComment)
+void TDebugLoggerBase::internalCloseBlocks(TDBG_LOCATION_PROTO cAppCharP aBlockName, cAppCharP aCloseComment)
 {
   if (!fDbgOptionsP) return; // security
   bool withTime = fDbgOptionsP->fTimestampStructure;
@@ -1077,7 +1208,7 @@ void TDebugLoggerBase::internalCloseBlocks(cAppCharP aBlockName, cAppCharP aClos
   #if SYDEBUG>1
   if (!fBlockHistory && aBlockName) {
     // no blocks open any more and not close-all-remaining call (log close...)
-    DebugPrintf(DBG_EXOTIC+DBG_ERROR,"Block Nest Warning: Trying to close block '%s', but no block is open",aBlockName);
+    DebugPrintf(TDBG_LOCATION_ARG DBG_EXOTIC+DBG_ERROR,"Block Nest Warning: Trying to close block '%s', but no block is open",aBlockName);
   }
   #endif
   while (fBlockHistory) {
@@ -1100,13 +1231,13 @@ void TDebugLoggerBase::internalCloseBlocks(cAppCharP aBlockName, cAppCharP aClos
     }
     // now close topmost Block
     string ts,bl;
-    // - get time if needed and eventually put it within indented block
+    // - get time if needed and possibly put it within indented block
     if (withTime) {
       StringObjTimestamp(ts,getSystemNowAs(TCTX_SYSTEM));
       // for XML, the time must be shown before the close tag on a separate line
       if (fDbgOptionsP->fOutputFormat == dbgfmt_xml) {
         StringObjPrintf(bl,"<endblock time=\"%s\"/>",ts.c_str());
-        DebugPutLine(bl.c_str(), bl.size()); // still within block, indented
+        DebugPutLine(TDBG_LOCATION_NONE bl.c_str(), bl.size()); // still within block, indented
       }
     }
     // - now unindent
@@ -1138,7 +1269,7 @@ void TDebugLoggerBase::internalCloseBlocks(cAppCharP aBlockName, cAppCharP aClos
         }
         StringObjAppendPrintf(bl,"<a name=\"F%ld\">",long(fBlockHistory->fBlockNo));
         if (withTime) {
-          bl+="[" + ts + "] ";
+          bl += MAKEDBGLINK(string("[") + ts + "] ");
         }
         bl += "End of '";
         bl+=fBlockHistory->fBlockName;
@@ -1170,7 +1301,7 @@ void TDebugLoggerBase::internalCloseBlocks(cAppCharP aBlockName, cAppCharP aClos
         break;
     } // switch Block close
     // - output closing Block line
-    DebugPutLine(bl.c_str(), bl.size());
+    DebugPutLine(TDBG_LOCATION_NONE bl.c_str(), bl.size());
     // - remove Block level
     TBlockLevel *closedLevel = fBlockHistory;
     fBlockHistory = closedLevel->fNext;
@@ -1212,10 +1343,10 @@ bool TDebugLoggerBase::DebugStartOutput(void)
         // 256 is a safe assumption because the "fold" button <divs> alone are around 250 bytes
         fBlockNo = 1 + (fDbgOutP->dbgFileSize()/256);
         // now create required prefix
-        DebugPutLine(fDbgOptionsP->fCustomPrefix.empty() ? DbgOutDefaultPrefixes[fDbgOptionsP->fOutputFormat] : fDbgOptionsP->fCustomPrefix.c_str());
+        DebugPutLine(TDBG_LOCATION_NONE fDbgOptionsP->fCustomPrefix.empty() ? DbgOutDefaultPrefixes[fDbgOptionsP->fOutputFormat] : fDbgOptionsP->fCustomPrefix.c_str());
         // add folding javascript if needed
         if (fDbgOptionsP->fOutputFormat==dbgfmt_html && fDbgOptionsP->fFoldingMode!=dbgfold_none) {
-          DebugPutLine(FoldingPrefix);
+          DebugPutLine(TDBG_LOCATION_NONE FoldingPrefix);
         }
       } // debug channel opened successfully
     } // use own debug channel
@@ -1229,17 +1360,17 @@ void TDebugLoggerBase::DebugFinalizeOutput(void)
 {
   if (fOutputLoggerP) {
   	// just close my own blocks
-    internalCloseBlocks(NULL,"closed because sub-log ends here");    
+    internalCloseBlocks(TDBG_LOCATION_NONE NULL,"closed because sub-log ends here");    
   }
   if (fOutStarted && fDbgOptionsP && fDbgOutP) {
     // close all left-open open Blocks
-    internalCloseBlocks(NULL,"closed because log ends here");
+    internalCloseBlocks(TDBG_LOCATION_NONE NULL,"closed because log ends here");
     // now finalize output
     // - special stuff before
     if (fDbgOptionsP->fOutputFormat == dbgfmt_xml)
       fIndent=0; // unindent to zero (document is not a real Block)
     // - then suffix
-    DebugPutLine(fDbgOptionsP->fCustomSuffix.empty() ? DbgOutDefaultSuffixes[fDbgOptionsP->fOutputFormat] : fDbgOptionsP->fCustomSuffix.c_str());
+    DebugPutLine(TDBG_LOCATION_NONE fDbgOptionsP->fCustomSuffix.empty() ? DbgOutDefaultSuffixes[fDbgOptionsP->fOutputFormat] : fDbgOptionsP->fCustomSuffix.c_str());
     // now close the debug channel
     fDbgOutP->closeDbg();
   }
@@ -1249,7 +1380,7 @@ void TDebugLoggerBase::DebugFinalizeOutput(void)
 
 
 // Output single line to debug channel (includes indenting and other prefixing, but no further formatting)
-void TDebugLoggerBase::DebugPutLine(cAppCharP aText, stringSize aTextSize, bool aPre)
+void TDebugLoggerBase::DebugPutLine(TDBG_LOCATION_PROTO cAppCharP aText, stringSize aTextSize, bool aPre)
 {
   if (!aText || (!fDbgOutP && !fOutputLoggerP)) return;
   if (*aText) {
@@ -1379,7 +1510,7 @@ TDebugLoggerBase *TDebugLogger::getThreadLogger(bool aCreateNew)
   // unknown subthread
   if (fMainThreadID==0) {
     // no current mainthread, let subthread write to main log
-    // Note: this makes sure log info eventually trailing the DebugThreadOutputDone()
+    // Note: this makes sure log info possibly trailing the DebugThreadOutputDone()
     //       also lands in the main log. This is not critical - the only thing that must be
     //       ensured is that starting new threads is made only with DebugDefineMainThread set.
     return this;
@@ -1445,31 +1576,31 @@ TDebugLoggerBase &TDebugLogger::setNextMask(uInt32 aDbgMask)
 
 
 // output text to debug channel, with checking for subthreads
-void TDebugLogger::DebugPuts(uInt32 aDbgMask, cAppCharP aText, stringSize aTextSize, bool aPreFormatted)
+void TDebugLogger::DebugPuts(TDBG_LOCATION_PROTO uInt32 aDbgMask, cAppCharP aText, stringSize aTextSize, bool aPreFormatted)
 {
   TDebugLoggerBase *loggerP = getThreadLogger();
-  if (loggerP) loggerP->inherited::DebugPuts(aDbgMask,aText,aTextSize,aPreFormatted);
+  if (loggerP) loggerP->inherited::DebugPuts(TDBG_LOCATION_ARG aDbgMask,aText,aTextSize,aPreFormatted);
 } // TDebugLogger::DebugPuts
 
 
-void TDebugLogger::DebugVPrintf(uInt32 aDbgMask, cAppCharP aFormat, va_list aArgs)
+void TDebugLogger::DebugVPrintf(TDBG_LOCATION_PROTO uInt32 aDbgMask, cAppCharP aFormat, va_list aArgs)
 {
   TDebugLoggerBase *loggerP = getThreadLogger();
-  if (loggerP) loggerP->inherited::DebugVPrintf(aDbgMask,aFormat,aArgs);
+  if (loggerP) loggerP->inherited::DebugVPrintf(TDBG_LOCATION_ARG aDbgMask,aFormat,aArgs);
 } // TDebugLogger::DebugVPrintf
 
 
-void TDebugLogger::DebugVOpenBlock(cAppCharP aBlockName, cAppCharP aBlockTitle, bool aCollapsed, cAppCharP aBlockFmt, va_list aArgs)
+void TDebugLogger::DebugVOpenBlock(TDBG_LOCATION_PROTO cAppCharP aBlockName, cAppCharP aBlockTitle, bool aCollapsed, cAppCharP aBlockFmt, va_list aArgs)
 {
   TDebugLoggerBase *loggerP = getThreadLogger();
-  if (loggerP) loggerP->inherited::DebugVOpenBlock(aBlockName, aBlockTitle, aCollapsed, aBlockFmt, aArgs);
+  if (loggerP) loggerP->inherited::DebugVOpenBlock(TDBG_LOCATION_ARG aBlockName, aBlockTitle, aCollapsed, aBlockFmt, aArgs);
 } // TDebugLogger::DebugVOpenBlock
 
 
-void TDebugLogger:: DebugCloseBlock(cAppCharP aBlockName)
+void TDebugLogger:: DebugCloseBlock(TDBG_LOCATION_PROTO cAppCharP aBlockName)
 {
   TDebugLoggerBase *loggerP = getThreadLogger();
-  if (loggerP) loggerP->inherited::DebugCloseBlock(aBlockName);
+  if (loggerP) loggerP->inherited::DebugCloseBlock(TDBG_LOCATION_ARG aBlockName);
 } // TDebugLogger::DebugCloseBlock
 
 #endif
@@ -1517,7 +1648,7 @@ void TDebugLogger::DebugThreadOutputDone(bool aRemoveIt)
 
 
 // Used to regain control as main thread (e.g. for the next request of a session which
-// eventually occurs from another thread).
+// possibly occurs from another thread).
 void TDebugLogger::DebugDefineMainThread(void)
 {
   #ifdef MULTI_THREAD_SUPPORT
